@@ -4,6 +4,7 @@ import importlib
 import numpy as np
 import pandas as pd
 import json
+from joblib import load
 from jsonschema import validate
 from sklearn.pipeline import make_pipeline
 from timeflux.core.node import Node
@@ -40,7 +41,7 @@ class Pipeline(Node):
         o_events (Port): Event output, provides DataFrame.
 
     Args:
-        steps (dict): Pipeline steps and settings
+        steps (dict): Pipeline steps and settings (ignored if 'model' is set)
         fit (bool):
         mode ('predict'|'predict_proba'|'predict_log_proba'|'transform'|'fit_predict'|'fit_transform'):
         meta_label (str|tuple|None):
@@ -53,14 +54,15 @@ class Pipeline(Node):
         resample (bool):
         resample_direction ('right'|'left'|'both'):
         resample_rate (None|float):
-        model: Load a pickle model - NOT IMPLEMENTED
+        warmup (str): Load a .npy or .npz file and bootstrap the model with initial data
+        model (str): Load a pre-computed model, persisted with joblib
         cv: Cross-validation - NOT IMPLEMENTED
 
     """
 
     def __init__(
         self,
-        steps,
+        steps=None,
         fit=True,
         mode="predict",
         meta_label=("epoch", "context", "target"),
@@ -73,6 +75,7 @@ class Pipeline(Node):
         resample=False,
         resample_direction="right",
         resample_rate=None,
+        warmup=None,
         model=None,
         cv=None,
     ):
@@ -92,8 +95,15 @@ class Pipeline(Node):
         self.resample = resample
         self.resample_direction = resample_direction
         self.resample_rate = resample_rate
+        self.warmup = warmup
+        self.model = model
         self._buffer_size = pd.Timedelta(buffer_size)
-        self._make_pipeline(steps)
+        if model:
+            self._load_pipeline(model)
+        elif steps:
+            self._make_pipeline(steps)
+        else:
+            raise ValueError("You must pass either a 'steps' or 'model' argument")
         self._reset()
 
     def update(self):
@@ -148,6 +158,7 @@ class Pipeline(Node):
             if match_events(self.i_events, self.event_start_training) is not None:
                 self._status = FITTING
                 self.logger.debug("Start training")
+                self._warmup()
                 self._task = Task(
                     self._pipeline, "fit", self._X_train, self._y_train
                 ).start()
@@ -160,7 +171,6 @@ class Pipeline(Node):
                     self._pipeline = status["instance"]
                     self._status = READY
                     self.logger.debug(f"Model fitted in {status['time']} seconds")
-                    # TODO: this can potentially be overwritten in _send()
                     self.o_events.data = make_event("ready")
                 else:
                     self.logger.error(
@@ -205,6 +215,10 @@ class Pipeline(Node):
             self.fit = False
         elif self.mode.startswith("predict"):
             self.fit = True
+        if self.model is not None:
+            self.fit = False
+            if not self.mode.startswith("fit"):
+                self.meta_label = None
         if self.fit:
             self._status = IDLE
         else:
@@ -260,6 +274,44 @@ class Pipeline(Node):
         # TODO: memory and verbose args
         self._pipeline = make_pipeline(*pipeline, memory=None, verbose=False)
 
+    def _load_pipeline(self, path):
+
+        try:
+            self._pipeline = load(path)
+        except:
+            self.logger.error("Could not load model")
+            raise WorkerInterrupt()
+
+    def _warmup(self):
+
+        if self.warmup:
+            try:
+                data = np.load(self.warmup)
+                if type(data) == np.ndarray:
+                    data = {
+                        "X": data
+                    }  # .npy return an ndarray while .npz return a dict
+                if "X" in data:
+                    if self._X_train is None:
+                        self._X_train = data["X"]
+                    else:
+                        self._X_train = np.vstack((data["X"], self._X_train))
+                else:
+                    self.logger.warning("Warmup data is missing")
+                if "y" in data:
+                    if self._y_train is None:
+                        self._y_train = data["y"]
+                    else:
+                        self._y_train = np.append(data["y"], self._y_train)
+                else:
+                    self.logger.info("Warmup labels are missing")  # OK if unsupervised
+            except OSError:
+                self.logger.error("Warmup file does not exist or cannot be read")
+                raise WorkerInterrupt()
+            except ValueError:
+                self.logger.error("Warmup and training data dimensions do not match")
+                raise WorkerInterrupt()
+
     def _accumulate(self, start, stop):
 
         # Do nothing if no fitting required
@@ -403,7 +455,15 @@ class Pipeline(Node):
                         if self._dimensions == 2
                         else {"epochs": self._X_meta}
                     )  # port.meta should always be an object
-                    self.o_events.set(data, times, names, meta)
+                    if hasattr(self._pipeline, "classes_"):
+                        meta["classes"] = list(self._pipeline.classes_)
+                    rows = pd.DataFrame(data, index=times, columns=names)
+                    if self.o_events.ready():
+                        # Make sure we don't overwrite other events
+                        self.o_events.data = pd.concat([self.o_events.data, rows])
+                    else:
+                        self.o_events.data = rows
+                    self.o_events.meta = meta
                 else:
                     self.logger.warning(
                         "Number of predictions inconsistent with input length"
